@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package:CL-WHO; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-who/who.lisp,v 1.10 2003/08/02 09:17:49 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-who/who.lisp,v 1.16 2004/04/24 00:05:39 edi Exp $
 
-;;; Copyright (c) 2003, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2003-2004, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -121,20 +121,34 @@ character set."
   (declare (optimize speed space))
   "Returns a string list corresponding to the HTML tag TAG. Utility
 function used by TREE-TO-TEMPLATE."
-  (multiple-value-bind (attr-list body)
-      ;; collect attribute/value pairs into ATTR-LIST and tag body (if
-      ;; any) into BODY
-      (loop for rest on (cdr list) by #'cddr
-            if (keywordp (first rest))
-              collect (cons (first rest) (second rest)) into attr-list
-            else do (return (values attr-list rest))
-            finally (return attr-list))
+  (let (tag attr-list body)
+    (cond
+      ((atom (first list))
+       (setq tag (first list))
+       ;; collect attribute/value pairs into ATTR-LIST and tag body (if
+       ;; any) into BODY
+       (loop for rest on (cdr list) by #'cddr
+	     if (keywordp (first rest))
+	       collect (cons (first rest) (second rest)) into attr
+	     else
+	       do (progn (setq attr-list attr)
+			 (setq body rest)
+			 (return))
+	     finally (setq attr-list attr)))
+      ((listp (first list))
+       (setq tag (first (first list)))
+       (loop for rest on (cdr (first list)) by #'cddr
+	     if (keywordp (first rest))
+	     collect (cons (first rest) (second rest)) into attr
+	     finally (setq attr-list attr))
+       (setq body (cdr list))))
+
     (nconc
      (if *indent*
        ;; indent by *INDENT* spaces
        (list +newline+ (n-spaces *indent*)))
      ;; tag name
-     (list "<" (string-downcase (first list)))
+     (list "<" (string-downcase tag))
      ;; attributes
      (loop with =var= = (gensym)
            for (attr . val) in attr-list
@@ -182,7 +196,7 @@ function used by TREE-TO-TEMPLATE."
           ;; indentation
           (list +newline+ (n-spaces *indent*)))
         ;; closing tag
-        (list "</" (string-downcase (first list)) ">"))
+        (list "</" (string-downcase tag) ">"))
        ;; no body, so no closing tag
        (list " />")))))
 
@@ -226,8 +240,12 @@ flattened list of strings. Utility function used by TREE-TO-COMMANDS-AUX."
                                          "<"
                                          (string-downcase element)
                                          " />")))
-                    ((and (listp element)
-                          (keywordp (first element)))
+                    ((or
+		      (and (listp element)
+			   (keywordp (first element)))
+		      (and (listp element)
+			   (listp (first element))
+			   (keywordp (first (first element)))))
                       ;; normal tag
                       (print-tag element))
                     ((listp element)
@@ -390,3 +408,22 @@ into Lisp code which creates the corresponding HTML as a string."
     (setq prologue *prologue*))
   `(pprint '(let ((,var ,(or stream var)))
              ,(tree-to-commands body var prologue))))
+
+;; stuff for Nikodemus Siivola's HYPERDOC
+;; see <http://common-lisp.net/project/hyperdoc/>
+;; and <http://www.cliki.net/hyperdoc>
+
+(defvar *hyperdoc-base-uri* "http://weitz.de/cl-who/")
+
+(let ((exported-symbols-alist
+       (loop for symbol being the external-symbols of :cl-who
+             collect (cons symbol
+                           (concatenate 'string
+                                        "#"
+                                        (string-downcase symbol))))))
+  (defun hyperdoc-lookup (symbol type)
+    (declare (ignore type))
+    (cdr (assoc symbol
+                exported-symbols-alist
+                :test #'eq))))
+               
\ No newline at end of file
